searchData={"items":[{"type":"module","title":"Nebulex.Adapters.Local","doc":"A Local Generation Cache adapter for Nebulex; inspired by\n[epocxy cache](https://github.com/duomark/epocxy/blob/master/src/cxy_cache.erl).\n\nGenerational caching using an ETS table (or multiple ones when used with\n`:shards`) for each generation of cached data. Accesses hit the newer\ngeneration first, and migrate from the older generation to the newer\ngeneration when retrieved from the stale table. When a new generation\nis started, the oldest one is deleted. This is a form of mass garbage\ncollection which avoids using timers and expiration of individual\ncached elements.\n\nThis implementation of generation cache uses only two generations, referred\nto as the `new` and the `old` generation.\n\nSee `Nebulex.Adapters.Local.Generation` to learn more about generation\nmanagement and garbage collection.","ref":"Nebulex.Adapters.Local.html"},{"type":"module","title":"Overall features - Nebulex.Adapters.Local","doc":"* Configurable backend (`ets` or `:shards`).\n  * Expiration - A status based on TTL (Time To Live) option. To maintain\n    cache performance, expired entries may not be immediately removed or\n    evicted, they are expired or evicted on-demand, when the key is read.\n  * Eviction - [Generational Garbage Collection][gc].\n  * Sharding - For intensive workloads, the Cache may also be partitioned\n    (by using `:shards` backend and specifying the `:partitions` option).\n  * Support for transactions via Erlang global name registration facility.\n    See `Nebulex.Adapter.Transaction`.\n  * Support for stats.\n\n[gc]: http://hexdocs.pm/nebulex/3.0.0-rc.1/Nebulex.Adapters.Local.Generation.html","ref":"Nebulex.Adapters.Local.html#module-overall-features"},{"type":"module","title":"Configuration options - Nebulex.Adapters.Local","doc":"The following options can be used to configure the adapter:\n\n* `:cache` (`t:atom/0`) - Required. The defined cache module.\n\n* `:stats` (`t:boolean/0`) - A flag to determine whether to collect cache stats. The default value is `true`.\n\n* `:backend` (`t:backend/0`) - The backend or storage to be used for the adapter. The default value is `:ets`.\n\n* `:read_concurrency` (`t:boolean/0`) - Since the adapter uses ETS tables internally, this option is when creating\n  a new table or generation. See `:ets.new/2` options. The default value is `true`.\n\n* `:write_concurrency` (`t:boolean/0`) - Since the adapter uses ETS tables internally, this option is when creating\n  a new table or generation. See `:ets.new/2` options. The default value is `true`.\n\n* `:compressed` (`t:boolean/0`) - Since the adapter uses ETS tables internally, this option is when creating\n  a new table or generation. See `:ets.new/2` options. The default value is `false`.\n\n* `:backend_type` - Since the adapter uses ETS tables internally, this option is when creating\n  a new table or generation. See `:ets.new/2` options. The default value is `:set`.\n\n* `:partitions` (`t:pos_integer/0`) - The number of ETS partitions when using the `:shards` backend.\n  See `:shards.new/2`.\n\n  The default value is `System.schedulers_online()`.\n\n* `:purge_chunk_size` (`t:pos_integer/0`) - This option limits the max nested match specs based on the number of keys\n  when purging the older cache generation. The default value is `100`.\n\n* `:gc_interval` (`t:pos_integer/0`) - The interval time in milliseconds for garbage collection to run, create\n  a new generation, make it the newer one, make the previous new generation\n  the old one, and finally remove the previous old one. If not provided\n  (or `nil`), the garbage collection never runs, so new generations must be\n  created explicitly, e.g., `MyCache.new_generation(opts)` (the default);\n  however, the adapter does not recommend this.\n\n  > #### Usage {: .warning}\n  >\n  > Always provide the `:gc_interval` option so the garbage collector can\n  > work appropriately out of the box. Unless you explicitly want to turn\n  > off the garbage collection or handle it yourself.\n\n* `:max_size` (`t:pos_integer/0`) - The maximum number of entries to store in the cache. If not provided\n  (or `nil`), the health check to validate and release memory is not\n  performed (the default).\n\n* `:allocated_memory` (`t:pos_integer/0`) - The maximum size in bytes for the cache storage. If not provided\n  (or `nil`), the health check to validate and release memory is not\n  performed (the default).\n\n* `:gc_memory_check_interval` (`t:mem_check_interval/0`) - The interval time in milliseconds for garbage collection to run the size\n  and memory checks.\n\n  > #### Usage {: .warning}\n  >\n  > Beware: For the `:gc_memory_check_interval` option to work, you must\n  > configure one of `:max_size` or `:allocated_memory` (or both).\n\n  The default value is `10000`.\n\n* `:gc_flush_delay` (`t:pos_integer/0`) - The delay in milliseconds before objects from the oldest generation\n  are flushed. The default value is `10000`.","ref":"Nebulex.Adapters.Local.html#module-configuration-options"},{"type":"module","title":"Usage - Nebulex.Adapters.Local","doc":"`Nebulex.Cache` is the wrapper around the cache. We can define a\nlocal cache as follows:\n\n    defmodule MyApp.LocalCache do\n      use Nebulex.Cache,\n        otp_app: :my_app,\n        adapter: Nebulex.Adapters.Local\n    end\n\nWhere the configuration for the cache must be in your application\nenvironment, usually defined in your `config/config.exs`:\n\n    config :my_app, MyApp.LocalCache,\n      gc_interval: :timer.hours(12),\n      max_size: 1_000_000,\n      allocated_memory: 2_000_000_000,\n      gc_memory_check_interval: :timer.seconds(10)\n\nFor intensive workloads, the Cache may also be partitioned using `:shards`\nas cache backend (`backend: :shards`) and configuring the desired number of\npartitions via the `:partitions` option. Defaults to\n`System.schedulers_online()`.\n\n    config :my_app, MyApp.LocalCache,\n      backend: :shards,\n      gc_interval: :timer.hours(12),\n      max_size: 1_000_000,\n      allocated_memory: 2_000_000_000,\n      gc_memory_check_interval: :timer.seconds(10)\n      partitions: System.schedulers_online() * 2\n\nIf your application was generated with a supervisor (by passing `--sup`\nto `mix new`) you will have a `lib/my_app/application.ex` file containing\nthe application start callback that defines and starts your supervisor.\nYou just need to edit the `start/2` function to start the cache as a\nsupervisor on your application's supervisor:\n\n    def start(_type, _args) do\n      children = [\n        {MyApp.LocalCache, []},\n        ...\n      ]\n\nSee `Nebulex.Cache` for more information.","ref":"Nebulex.Adapters.Local.html#module-usage"},{"type":"module","title":"The `:ttl` option - Nebulex.Adapters.Local","doc":"The `:ttl` is a runtime option meant to set a key's expiration time. It is\nevaluated on-demand when a key is retrieved, and if it has expired, it is\nremoved from the cache. Hence, it can not be used as an eviction method;\nit is more for maintaining the cache's integrity and consistency. For this\nreason, you should always configure the eviction or GC options. See the\n[\"Eviction policy\"](#module-eviction-policy) section for more information.","ref":"Nebulex.Adapters.Local.html#module-the-ttl-option"},{"type":"module","title":"Caveats when using `:ttl` option: - Nebulex.Adapters.Local","doc":"* When using the `:ttl` option, ensure it is less than `:gc_interval`.\n    Otherwise, the key may be evicted, and the `:ttl` hasn't happened yet\n    because the garbage collector may run before a fetch operation has\n    evaluated the `:ttl` and expired the key.\n  * Consider the following scenario based on the previous caveat. You have\n    `:gc_interval` set to 1 hrs. Then you put a new key with `:ttl` set to\n    2 hrs. One minute later, the GC runs, creating a new generation, and the\n    key ends up in the older generation. Therefore, if the next GC cycle\n    occurs (1 hr later) before the key is fetched (moving it to the newer\n    generation), it is evicted from the cache when the GC removes the older\n    generation so it won't be retrievable anymore.","ref":"Nebulex.Adapters.Local.html#module-caveats-when-using-ttl-option"},{"type":"module","title":"Eviction policy - Nebulex.Adapters.Local","doc":"This adapter implements a generational cache, which means its primary eviction\nmechanism pushes a new cache generation and removes the oldest one. This\nmechanism ensures the garbage collector removes the least frequently used keys\nwhen it runs and deletes the oldest generation. At the same time, only the\nmost frequently used keys are always available in the newer generation. In\nother words, the generation cache also enforces an LRU (Least Recently Used)\neviction policy.\n\nThe following conditions trigger the garbage collector to run:\n\n  * When the time interval defined by `:gc_interval` is completed. This\n    makes the garbage-collector process to run creating a new generation\n    and forcing to delete the oldest one. This interval defines how often\n    you want to evict the least frequently used entries or the retention\n    period for the cached entries. The retention period for the least\n    frequently used entries is equivalent to two garbage collection cycles\n    (since we keep two generations), which means the GC removes all entries\n    not accessed in the cache during that time.\n\n  * When the time interval defined by `:gc_memory_check_interval` is\n    completed. Beware: This option works alongside the `:max_size` and\n    `:allocated_memory` options. The interval defines when the GC must\n    run to validate the cache size and memory and release space if any\n    of the limits are exceeded. It is mainly for keeping the cached data\n    under the configured memory size limits and avoiding running out of\n    memory at some point.","ref":"Nebulex.Adapters.Local.html#module-eviction-policy"},{"type":"module","title":"Configuring the GC options - Nebulex.Adapters.Local","doc":"This section helps you understand how the different configuration options work\nand gives you an idea of what values to set, especially if this is your first\ntime using Nebulex with the local adapter.\n\nUnderstanding a few things in advance is essential to configure the cache\nwith appropriate values. For example, the average size of an entry so we\ncan configure a reasonable value for the max size or allocated memory.\nAlso, the reads and writes load. The problem is that sometimes it is\nchallenging to have this information in advance, especially when it is\na new app or when we use the cache for the first time. The following\nare tips to help you to configure the cache (especially if it is your\nfor the first time):\n\n  * To configure the GC, consider the retention period for the least\n    frequently used entries you desire. For example, if the GC is 1 hr, you\n    will keep only those entries accessed periodically during the last 2 hrs\n    (two GC cycles, as outlined above). If it is your first time using the\n    local adapter, you may start configuring the `:gc_interval` to 12 hrs to\n    ensure daily data retention. Then, you can analyze the data and change the\n    value based on your findings.\n\n  * Configure the `:max_size` or `:allocated_memory` option (or both) to keep\n    memory healthy under the given limits (avoid running out of memory).\n    Configuring these options will ensure the GC releases memory space\n    whenever a limit is reached or exceeded. For example, one may assign 50%\n    of the total memory to the `:allocated_memory`. It depends on how much\n    memory you need and how much your app needs to run. For the `:max_size`,\n    consider how many entries you expect to keep in the cache; you could start\n    with something between `100_000` and `1_000_000`.\n\n  * Finally, when configuring `:max_size` or `:allocated_memory` (or both),\n    you must also configure `:gc_memory_check_interval` (defaults to 10 sec).\n    By default, the GC will run every 10 seconds to validate the cache size\n    and memory.","ref":"Nebulex.Adapters.Local.html#module-configuring-the-gc-options"},{"type":"module","title":"Queryable API - Nebulex.Adapters.Local","doc":"Since the adapter implementation uses ETS tables underneath, the query must be\na valid [**ETS Match Spec**](https://www.erlang.org/doc/man/ets#match_spec).\nHowever, there are some predefined or shorthand queries you can use. See the\n[\"Predefined queries\"](#module-predefined-queries) section for information.\n\nThe adapter defines an entry as a tuple `{:entry, key, value, touched, ttl}`,\nmeaning the match pattern within the ETS Match Spec must be like\n`{:entry, :\"$1\", :\"$2\", :\"$3\", :\"$4\"}`. To make query building easier,\nyou can use the `Ex2ms` library.\n\n    iex> match_spec = [\n    ...>   {\n    ...>     {:entry, :\"$1\", :\"$2\", :_, :_},\n    ...>     [{:>, :\"$2\", 1}],\n    ...>     [{{:\"$1\", :\"$2\"}}]\n    ...>   }\n    ...> ]\n    iex> MyCache.get_all(query: match_spec)\n    {:ok, [b: 1, c: 3]}\n\n> You can use the `Ex2ms` or `MatchSpec` library to build queries easier.","ref":"Nebulex.Adapters.Local.html#module-queryable-api"},{"type":"module","title":"Transaction API - Nebulex.Adapters.Local","doc":"This adapter inherits the default implementation provided by\n`Nebulex.Adapter.Transaction`. Therefore, the `transaction` command accepts\nthe following options:\n\n* `:keys` (list of `t:term/0`) - The list of keys the transaction will lock. Since the lock ID is generated\n  based on the key, the transaction uses a fixed lock ID if the option is\n  not provided or is an empty list. Then, all subsequent transactions\n  without this option (or set to an empty list) are serialized, and\n  performance is significantly affected. For that reason, it is recommended\n  to pass the list of keys involved in the transaction. The default value is `[]`.\n\n* `:nodes` (list of `t:atom/0`) - The list of the nodes where to set the lock.\n\n  The default value is `[node()]`.\n\n* `:retries` (`:infinity` | `t:non_neg_integer/0`) - If the key has already been locked by another process and retries are not\n  equal to 0, the process sleeps for a while and tries to execute the action\n  later. When `:retries` attempts have been made, an exception is raised. If\n  `:retries` is `:infinity` (the default), the function will eventually be\n  executed (unless the lock is never released). The default value is `:infinity`.","ref":"Nebulex.Adapters.Local.html#module-transaction-api"},{"type":"module","title":"Extended API (extra functions) - Nebulex.Adapters.Local","doc":"This adapter provides some additional convenience functions to the\n`Nebulex.Cache` API.\n\nCreating new generations:\n\n    MyCache.new_generation()\n    MyCache.new_generation(gc_interval_reset: false)\n\nRetrieving the current generations:\n\n    MyCache.generations()\n\nRetrieving the newer generation:\n\n    MyCache.newer_generation()","ref":"Nebulex.Adapters.Local.html#module-extended-api-extra-functions"},{"type":"type","title":"Nebulex.Adapters.Local.backend/0","doc":"Adapter's backend type","ref":"Nebulex.Adapters.Local.html#t:backend/0"},{"type":"macro","title":"Nebulex.Adapters.Local.entry/1","doc":"","ref":"Nebulex.Adapters.Local.html#entry/1"},{"type":"macro","title":"Nebulex.Adapters.Local.entry/2","doc":"","ref":"Nebulex.Adapters.Local.html#entry/2"},{"type":"type","title":"Nebulex.Adapters.Local.mem_check_interval/0","doc":"The type for the `:gc_memory_check_interval` option value.\n\nThe `:gc_memory_check_interval` value can be:\n\n  * A positive integer with the time in milliseconds.\n  * An anonymous function to call in runtime and must return the next interval\n    in milliseconds. The function receives three arguments:\n    * The first argument is an atom indicating the limit, whether it is\n      `:size` or `:memory`.\n    * The second argument is the current value for the limit. For example,\n      if the limit in the first argument is `:size`, the second argument tells\n      the current cache size (number of entries in the cache). If the limit is\n      `:memory`, it means the recent cache memory in bytes.\n    * The third argument is the maximum limit provided in the configuration.\n      When the limit in the first argument is `:size`, it is the `:max_size`.\n      On the other hand, if the limit is `:memory`, it is the\n      `:allocated_memory`.","ref":"Nebulex.Adapters.Local.html#t:mem_check_interval/0"},{"type":"module","title":"Nebulex.Adapters.Local.Generation","doc":"This module implements the generation manager and garbage collector.\n\nThe generational garbage collector manages the heap as several sub-heaps,\nknown as generations, based on the age of the objects. An object is allocated\nin the youngest generation, sometimes called the nursery, and is promoted to\nan older generation if its lifetime exceeds the threshold of its current\ngeneration (defined by option `:gc_interval`). Every time the GC runs, a new\ncache generation is created, and the oldest one is deleted.\n\nThe GC is triggered upon the following events:\n\n  * When the `:gc_interval` times outs.\n  * When the scheduled memory check times out. The memory check is executed\n    when the `:max_size` or `:allocated_memory` options (or both) are\n    configured. Furthermore, the memory check interval is determined by\n    option `:gc_memory_check_interval`.\n\nThe oldest generation is deleted in two steps. First, the underlying ETS table\nis flushed to release space and only marked for deletion as there may still be\nprocesses referencing it. The actual deletion of the ETS table happens at the\nnext GC run. However, flushing is a blocking operation. Once started,\nprocesses wanting to access the table must wait until it finishes.\nTo circumvent this, the flush can be delayed by configuring `:gc_flush_delay`\nto allow time for these processes to complete their work without being\nblocked.","ref":"Nebulex.Adapters.Local.Generation.html"},{"type":"function","title":"Nebulex.Adapters.Local.Generation.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Nebulex.Adapters.Local.Generation.html#child_spec/1"},{"type":"function","title":"Nebulex.Adapters.Local.Generation.delete_all/1","doc":"Removes or flushes all entries from the cache (including all its generations).","ref":"Nebulex.Adapters.Local.Generation.html#delete_all/1"},{"type":"function","title":"Example - Nebulex.Adapters.Local.Generation.delete_all/1","doc":"Nebulex.Adapters.Local.Generation.delete_all(MyCache)","ref":"Nebulex.Adapters.Local.Generation.html#delete_all/1-example"},{"type":"function","title":"Nebulex.Adapters.Local.Generation.get_state/1","doc":"A convenience function for retrieving the state.","ref":"Nebulex.Adapters.Local.Generation.html#get_state/1"},{"type":"function","title":"Nebulex.Adapters.Local.Generation.list/1","doc":"Returns the list of the generations in the form `[newer, older]`.","ref":"Nebulex.Adapters.Local.Generation.html#list/1"},{"type":"function","title":"Example - Nebulex.Adapters.Local.Generation.list/1","doc":"Nebulex.Adapters.Local.Generation.list(MyCache)","ref":"Nebulex.Adapters.Local.Generation.html#list/1-example"},{"type":"function","title":"Nebulex.Adapters.Local.Generation.memory_info/1","doc":"Returns the memory info in a tuple form `{used_mem, total_mem}`.","ref":"Nebulex.Adapters.Local.Generation.html#memory_info/1"},{"type":"function","title":"Example - Nebulex.Adapters.Local.Generation.memory_info/1","doc":"Nebulex.Adapters.Local.Generation.memory_info(MyCache)","ref":"Nebulex.Adapters.Local.Generation.html#memory_info/1-example"},{"type":"function","title":"Nebulex.Adapters.Local.Generation.new/2","doc":"Creates a new cache generation. Once the max number of generations\nis reached, when a new generation is created, the oldest one is\ndeleted.","ref":"Nebulex.Adapters.Local.Generation.html#new/2"},{"type":"function","title":"Options - Nebulex.Adapters.Local.Generation.new/2","doc":"* `:gc_interval_reset` (`t:boolean/0`) - Whether the `:gc_interval` should be reset or not. The default value is `true`.","ref":"Nebulex.Adapters.Local.Generation.html#new/2-options"},{"type":"function","title":"Example - Nebulex.Adapters.Local.Generation.new/2","doc":"Nebulex.Adapters.Local.Generation.new(MyCache)\n\n    Nebulex.Adapters.Local.Generation.new(MyCache, gc_interval_reset: false)","ref":"Nebulex.Adapters.Local.Generation.html#new/2-example"},{"type":"function","title":"Nebulex.Adapters.Local.Generation.newer/1","doc":"Returns the newer generation.","ref":"Nebulex.Adapters.Local.Generation.html#newer/1"},{"type":"function","title":"Example - Nebulex.Adapters.Local.Generation.newer/1","doc":"Nebulex.Adapters.Local.Generation.newer(MyCache)","ref":"Nebulex.Adapters.Local.Generation.html#newer/1-example"},{"type":"type","title":"Nebulex.Adapters.Local.Generation.opts/0","doc":"","ref":"Nebulex.Adapters.Local.Generation.html#t:opts/0"},{"type":"function","title":"Nebulex.Adapters.Local.Generation.realloc/2","doc":"Reallocates the block of memory that was previously allocated for the given\n`server_ref` with the new `size`. In other words, reallocates the max memory\nsize for a cache generation.","ref":"Nebulex.Adapters.Local.Generation.html#realloc/2"},{"type":"function","title":"Example - Nebulex.Adapters.Local.Generation.realloc/2","doc":"Nebulex.Adapters.Local.Generation.realloc(MyCache, 1_000_000)","ref":"Nebulex.Adapters.Local.Generation.html#realloc/2-example"},{"type":"function","title":"Nebulex.Adapters.Local.Generation.reset_gc_interval/1","doc":"Resets the GC interval.","ref":"Nebulex.Adapters.Local.Generation.html#reset_gc_interval/1"},{"type":"function","title":"Example - Nebulex.Adapters.Local.Generation.reset_gc_interval/1","doc":"Nebulex.Adapters.Local.Generation.reset_gc_interval(MyCache)","ref":"Nebulex.Adapters.Local.Generation.html#reset_gc_interval/1-example"},{"type":"function","title":"Nebulex.Adapters.Local.Generation.server/1","doc":"Returns the PID of the GC server for the given `server_ref`.","ref":"Nebulex.Adapters.Local.Generation.html#server/1"},{"type":"function","title":"Example - Nebulex.Adapters.Local.Generation.server/1","doc":"Nebulex.Adapters.Local.Generation.server(MyCache)","ref":"Nebulex.Adapters.Local.Generation.html#server/1-example"},{"type":"type","title":"Nebulex.Adapters.Local.Generation.server_ref/0","doc":"","ref":"Nebulex.Adapters.Local.Generation.html#t:server_ref/0"},{"type":"function","title":"Nebulex.Adapters.Local.Generation.start_link/1","doc":"Starts the garbage collector for the built-in local cache adapter.","ref":"Nebulex.Adapters.Local.Generation.html#start_link/1"},{"type":"type","title":"Nebulex.Adapters.Local.Generation.t/0","doc":"","ref":"Nebulex.Adapters.Local.Generation.html#t:t/0"},{"type":"module","title":"Nebulex.Adapters.Local.Options","doc":"Option definitions for the local adapter.","ref":"Nebulex.Adapters.Local.Options.html"},{"type":"function","title":"Nebulex.Adapters.Local.Options.adapter_options_docs/0","doc":"","ref":"Nebulex.Adapters.Local.Options.html#adapter_options_docs/0"},{"type":"function","title":"Nebulex.Adapters.Local.Options.gc_runtime_options_docs/0","doc":"","ref":"Nebulex.Adapters.Local.Options.html#gc_runtime_options_docs/0"},{"type":"function","title":"Nebulex.Adapters.Local.Options.validate_adapter_opts!/1","doc":"","ref":"Nebulex.Adapters.Local.Options.html#validate_adapter_opts!/1"},{"type":"function","title":"Nebulex.Adapters.Local.Options.validate_gc_runtime_opts!/1","doc":"","ref":"Nebulex.Adapters.Local.Options.html#validate_gc_runtime_opts!/1"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}